#Variables
# name = "Kenny"
# number = 10
# print(number + 20)

# number = 20
# if number < 10:
#     print("Number is larger than 10")
# elif 10 <= number <= 19:
#     print("Number is between 10 and 19")
# else:
#     print("Number is less than 10")

# for i in range(5, 12, 2):
#     print(i)

# while True:
#     print("Forever")

# numbers = [1, 3, 5, 7]
# floats = [2.5, 1.2, 9.999, "Kenny"]

# for however in range(len(floats)):
#     print(floats[however])

# for i in floats:
#     print(i)

#Function/Methods
# def Add(num1, num2):
#     return num1 + num2
# Add(5, 7)
#
# def GiveMeFive():
#     print(5)
# GiveMeFive()

# class BankAccount():
#     def __init__(self):
#         self.money = 0
#         self.type = "Chequing"
#
#     def deposit(self, amount):
#         self.money += amount


# bank_account = BankAccount()
# bank_account2 = BankAccount()
# bank_account2.money = 2000
# bank_account.deposit(1000)
# print(bank_account.money)
# print(bank_account2.money)




